package sun.misc;

import java.nio.ByteBuffer;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public abstract class CharacterDecoder
{
    public CharacterDecoder() {
    }
    
    protected abstract int bytesPerAtom();
    
    protected abstract int bytesPerLine();
    
    protected void decodeBufferPrefix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
    }
    
    protected void decodeBufferSuffix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
    }
    
    protected int decodeLinePrefix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
        return this.bytesPerLine();
    }
    
    protected void decodeLineSuffix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
    }
    
    protected void decodeAtom(PushbackInputStream pushbackInputStream, OutputStream outputStream, int n) throws IOException {
        throw new CEStreamExhausted();
    }
    
    protected int readFully(InputStream inputStream, byte[] array, int n, int n2) throws IOException {
        int i = 0;
        while (i < n2) {
            int read = inputStream.read();
            if (read == -1) {
                return (i == 0) ? -1 : i;
            }
            array[i + n] = (byte)read;
            ++i;
        }
        return n2;
    }
    
    public void decodeBuffer(InputStream in, OutputStream outputStream) throws IOException {
        int n = 0;
        PushbackInputStream pushbackInputStream = new PushbackInputStream(in);
        this.decodeBufferPrefix(pushbackInputStream, outputStream);
        while (true) {
            try {
                int decodeLinePrefix;
                int n2;
                for (decodeLinePrefix = this.decodeLinePrefix(pushbackInputStream, outputStream), n2 = 0; n2 + this.bytesPerAtom() < decodeLinePrefix; n += this.bytesPerAtom(), n2 += this.bytesPerAtom()) {
                    this.decodeAtom(pushbackInputStream, outputStream, this.bytesPerAtom());
                }
                if (n2 + this.bytesPerAtom() == decodeLinePrefix) {
                    this.decodeAtom(pushbackInputStream, outputStream, this.bytesPerAtom());
                    n += this.bytesPerAtom();
                }
                else {
                    this.decodeAtom(pushbackInputStream, outputStream, decodeLinePrefix - n2);
                    n += decodeLinePrefix - n2;
                }
                this.decodeLineSuffix(pushbackInputStream, outputStream);
                continue;
            }
            catch (CEStreamExhausted ceStreamExhausted) {
                this.decodeBufferSuffix(pushbackInputStream, outputStream);
                return;
            }
        }
    }
    
    public byte[] decodeBuffer(String s) throws IOException {
        byte[] array = new byte[s.length()];
        s.getBytes(0, s.length(), array, 0);
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(array);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        this.decodeBuffer(byteArrayInputStream, byteArrayOutputStream);
        return byteArrayOutputStream.toByteArray();
    }
    
    public byte[] decodeBuffer(InputStream inputStream) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        this.decodeBuffer(inputStream, byteArrayOutputStream);
        return byteArrayOutputStream.toByteArray();
    }
    
    public ByteBuffer decodeBufferToByteBuffer(String s) throws IOException {
        return ByteBuffer.wrap(this.decodeBuffer(s));
    }
    
    public ByteBuffer decodeBufferToByteBuffer(InputStream inputStream) throws IOException {
        return ByteBuffer.wrap(this.decodeBuffer(inputStream));
    }
}
